@@ -19,13 +19,16 @@ import warnings
 from statistics import median
 from SigProfilerMatrixGenerator.scripts import SigProfilerMatrixGeneratorFunc as matGen
 import time
+import pandas as pd
+import matplotlib.patches as mplpatches
+import pickle
 
 
 
 
 
 
-def plot96_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, sample, percentage, signature, panel1, panel3, panel5, fig):
+def plot96_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, sample, percentage, signature, panel1, panel3, panel5, fig, chrom=None):
 	if 'roman' in matplotlib.font_manager.weight_dict:
 		del matplotlib.font_manager.weight_dict['roman']
 		matplotlib.font_manager._rebuild()
@@ -106,7 +109,6 @@ def plot96_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, s
 			  	  ytick_offest*3, ytick_offest*4]		
 
 	labs = np.arange(0.375,96.375,1)
-
 	panel1.set_xlim([0, 96])
 	panel1.set_ylim([0, y])
 	panel1.set_xticks(labs)
@@ -164,7 +166,6 @@ def plot96_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, s
 
 	total_count = sum(sum(nuc.values()) for nuc in mutations[sample].values())
 	xlabels = []
-
 	x = 0.4
 	ymax = 0
 	colors = [[3/256,189/256,239/256], [1/256,1/256,1/256],[228/256,41/256,38/256], [203/256,202/256,202/256], [162/256,207/256,99/256], [236/256,199/256,197/256]]
@@ -206,7 +207,6 @@ def plot96_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, s
 
 	panel3.set_ylabel("Clustered", fontname='Times New Roman', fontsize=15, fontweight='bold')
 
-
 	while y%4 != 0:
 		y += 1
 	ytick_offest = int(y/4)
@@ -248,7 +248,6 @@ def plot96_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, s
 	panel3.grid(which='major', axis='y', color=[0.93,0.93,0.93], zorder=1)
 	[i.set_color("black") for i in panel3.get_yticklabels()]
 
-
 	#############################################################################################
 	# Plot 96 bar plot (non-clustered)
 	#############################################################################################
@@ -338,7 +337,6 @@ def plot96_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, s
 	panel5.set_xticks(labs)
 	panel5.set_yticks(ylabs)
 	panel5.grid(which='major', axis='y', color=[0.93,0.93,0.93], zorder=1)
-
 	count = 0
 	m = 0
 	for i in range (0, 96, 1):
@@ -358,11 +356,12 @@ def plot96_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, s
 					   direction='in', length=10, colors='lightgray', width=1)
 
 	[i.set_color("black") for i in panel5.get_yticklabels()]
-
-	fig.suptitle(sample, fontsize=30, fontname="Times New Roman")
+	if chrom:
+		fig.suptitle(sample + "(" + chrom + ")", fontsize=30, fontname="Times New Roman")
+	else:
+		fig.suptitle(sample, fontsize=30, fontname="Times New Roman")
 	plt.text(.008, .55, 'Mutation Counts', rotation='vertical', fontsize=20, fontweight='bold', fontname="Times New Roman", transform=plt.gcf().transFigure)
 
-
 def plotINDEL_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, sample, percentage, signature, panel1, panel3, panel5, fig):
 	if 'roman' in matplotlib.font_manager.weight_dict:
 		del matplotlib.font_manager.weight_dict['roman']
@@ -928,4 +927,376 @@ def plotINDEL_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered
 					   top=False, labeltop=False,\
 					   direction='in', length=25, colors='gray', width=2)
 	panel5.set_ylabel("Non-Clustered", fontname='Times New Roman', fontsize=15, fontweight='bold')
-	[i.set_color("black") for i in panel5.get_yticklabels()]
\ No newline at end of file
+	[i.set_color("black") for i in panel5.get_yticklabels()]
+
+
+
+
+
+def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 10000000):
+	aggregated = False
+	chrom_based = False
+
+	path_suffix = ''
+	if chrom_based_IMD:
+		path_suffix = '_chrom'
+	projectPath_parent2 = project_path
+	projectPath_parent = project_path
+	projectPath = project_path + "output/vcf_files/ESCC_clustered/subclasses" + path_suffix + "/"
+	outputPath = projectPath_parent + "output/plots/"
+
+	if not os.path.exists(outputPath):
+		os.makedirs(outputPath)
+
+	plot_suffix = project
+	if chrom_based:
+		plot_suffix += "_chrom_based"
+
+	if chrom_based_IMD:
+		plot_suffix += "_chrom"
+
+	classes = ["Class IA", "Class IB", "Class IC", "Class II", "Class III", "Non-clust"]
+	mutationsPath = [projectPath + "class1a/ESCC_clustered_class1a.txt", projectPath + "class1b/ESCC_clustered_class1b.txt", projectPath + "class1c/ESCC_clustered_class1c.txt",
+					projectPath + "class2/ESCC_clustered_class2.txt", projectPath + "class3/ESCC_clustered_class3.txt", projectPath_parent + "output/vcf_files/ESCC_nonClustered/SNV/ESCC_nonClustered.txt"]
+
+	with open(projectPath_parent2 + "output/simulations/data/imds" + path_suffix + ".pickle", "rb") as handle:
+		imdsData = pickle.load(handle) 
+
+	with open(projectPath_parent2 + "output/simulations/data/imds.pickle", "rb") as handle:
+		imdsDataSample = pickle.load(handle) 
+
+
+	headerFields = ["project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "vaf","class", "failedReason"]
+
+	centromeres = {'GRCh38':{'1': [122026460,125184587],'10': [39686683,41593521],'11':[51078349,54425074],'12':[34769408,37185252],'13': [16000001,18051248],
+				'14': [16000001,18173523],'15': [17000001,19725254],'16': [36311159,38280682],'17': [22813680,26885980],'18': [15460900,20861206],
+				'19': [24498981,27190874],'2': [92188146,94090557],'20': [26436233,30038348],'21': [10864561,12915808],'22': [12954789,15054318],
+				'3': [90772459,93655574],'4': [49708101,51743951],'5': [46485901,50059807],'6': [58553889,59829934],'7': [58169654,60828234],
+				'8': [44033745,45877265],'9': [43236168,45518558],'X': [58605580,62412542],'Y': [10316945,10544039]}, 
+
+				'GRCh37':{'1': [121535434,124535434],'10': [39254935,42254935],'11':[51644205,54644205],'12':[34856694,37856694],'13': [16000000,19000000],
+				'14': [16000000,19000000],'15': [17000000,20000000],'16': [35335801,38335801],'17': [22263006,25263006],'18': [15460898,18460898],
+				'19': [24681782,27681782],'2': [92326171,95326171],'20': [26369569,29369569],'21': [11288129,14288129],'22': [13000000,16000000],
+				'3': [90504854,93504854],'4': [49660117,52660117],'5': [46405641,49405641],'6': [58830166,61830166],'7': [58054331,61054331],
+				'8': [43838887,46838887],'9': [47367679,50367679],'X': [58632012,61632012],'Y': [10316945,10544039]},
+
+				'mm10':{'1': [110000,3000000],'10': [110000,3000000],'11':[110000,3000000],'12':[110000,3000000],'13': [110000,3000000],
+				'14': [110000,3000000],'15': [110000,3000000],'16': [110000,3000000],'17': [110000,3000000],'18': [110000,3000000],
+				'19': [110000,3000000],'2': [110000,3000000],'3': [110000,3000000],'4': [110000,3000000],'5': [110000,3000000],
+				'6': [110000,3000000],'7': [110000,3000000],'8': [110000,3000000],'9': [110000,3000000],'X': [110000,3000000],'Y': [110000,3000000]},
+
+				'mm9':{'1': [0,3000000],'10': [0,3000000],'11':[0,3000000],'12':[0,3000000],'13': [0,3000000],
+				'14': [0,3000000],'15': [0,3000000],'16': [0,3000000],'17': [0,3000000],'18': [0,3000000],
+				'19': [0,3000000],'2': [0,3000000],'3': [0,3000000],'4': [0,3000000],'5': [0,3000000],
+				'6': [0,3000000],'7': [0,3000000],'8': [0,3000000],'9': [0,3000000],'X': [0,3000000],'Y': [0,3000000]}}
+
+	chromLengths = {'GRCh37': {'1':249250619,'2':243199368,'3':198022427,'4':191154274,'5':180915259,'6':171115063,'7':159138662,
+							   '8':146364019,'9':141213427,'10':135534745,'11':135006514,'12':133851894,'13':115169878,'14':107349537,
+							   '15':102531391,'16':90354750,'17':81195205,'18':78077248,'19':59128982,'20':63025519,'21':48129895,
+							   '22':51304566,'X':155270559,'Y':59373566}}
+
+	# Gather all mutations based upon user input
+	samplesSet = set()
+	samples = {}
+	allMutations = {}
+	if classes[0] == "Class III":
+		mutations = pd.read_csv(mutationsPath[0], sep="\t", names=headerFields, header=None)
+	elif classes[0] == "Non-clust":
+		mutations = pd.read_csv(mutationsPath[0], sep="\t", names=headerFields[:-3], header=None, skiprows=[0])
+	elif classes[0] == "Simulation":
+		mutations = pd.read_csv(mutationsPath[0], sep="\t", names=["project", "ID","sim","genome","chr","start","end","strand","placeHolder","mutType", "ref", "alt1","alt", "placeHolder2","placeHolder3","samples", "seq"], header=None, skiprows=[0])
+	else:
+		mutations = pd.read_csv(mutationsPath[0], sep="\t", names=headerFields[:-1], header=None)
+	genome = mutations.head(1).loc[0,'genome']
+	mutations = mutations.set_index(['samples', 'chr'])
+	mutations = mutations.sort_index()
+	allMutations[classes[0]] = mutations
+	samples[classes[0]] = list(set(mutations.index.droplevel(1)))
+	samplesSet.update(list(set(mutations.index.droplevel(1))))
+	if len(mutationsPath) > 1:
+		for i in range(1, len(mutationsPath), 1):
+			if classes[i] == "Class III":
+				newMutations = pd.read_csv(mutationsPath[i], sep="\t", names=headerFields, header=None)
+			elif classes[i] == "Non-clust":
+				newMutations = pd.read_csv(mutationsPath[i], sep="\t", names=headerFields[:-3], header=None, skiprows=[0])
+			else:
+				newMutations = pd.read_csv(mutationsPath[i], sep="\t", names=headerFields[:-1], header=None)
+			newMutations = newMutations.set_index(['samples', 'chr'])
+			newMutations = newMutations.sort_index()
+			allMutations[classes[i]] = newMutations
+			samples[classes[i]] = list(set(newMutations.index.droplevel(1)))
+			samplesSet.update(list(set(newMutations.index.droplevel(1))))
+
+	# Set up chromosome parameters:
+	# chrom_path = "/anaconda3/lib/python3.6/site-packages/SigProfilerMatrixGenerator/references/chromosomes/tsb/" + genome + "/"
+	chroms = [x.split(".")[0] for x in os.listdir(chrom_path) if x != ".DS_Store" and x[0]!= "G" and x[0] != "M" and x != "BED_" + genome + "_proportions.txt"]
+	chroms = sorted(chroms, key = lambda x: ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22','X','Y'].index(x[0:]))
+	pp = PdfPages(outputPath + "rainfallPlots_clustered_" + plot_suffix + ".pdf")
+	colors = ['red', 'black', 'green', 'orange', 'blue', 'grey']
+	colors = {'Class IA':'red', "Class IB":'black', "Class IC":'green', "Class II":'orange', "Class III":'blue', "Non-clust":'grey', "Simulation":'grey'}
+
+	falsePositives = {}
+	falsePositives_1000 = {}
+
+	totalMutsIMD = {}
+	totalMuts1000 = {}
+	if not chrom_based:
+		genomeLength = sum([x for x in chromLengths[genome].values()])
+		xtick_pos = []
+		totalPos = 0
+		for chrom in chroms:
+			newPos = chromLengths[genome][chrom]
+			xtick_pos.append(newPos/2 + totalPos)
+			totalPos += newPos
+		bins = []
+		for i in range(0, genomeLength, windowSize):
+			bins.append(i)
+		bins.append(genomeLength)
+		count = 1
+		for sample in samplesSet:
+			falsePositives[sample.split("_")[0]] = []
+			falsePositives_1000[sample.split("_")[0]] = []
+			falsePositivesChrom = []
+			falsePositivesChrom_1000 = []
+			totalMutsIMDChrom = []
+			totalMuts1000Chrom = []
+			densityMuts = []
+			count += 1
+			plot1 = plt.figure(figsize=(12,8))
+			plt.rc('axes', edgecolor='lightgray')
+			panel1 = plt.axes([0.07, 0.09, 0.9, 0.6])
+			panel2 = plt.axes([0.07, 0.75, 0.9, 0.15])
+			if not aggregated and not chrom_based_IMD:
+				imd_line = imdsData[sample.split("_")[0]]
+				panel1.axhline(imd_line, color='red')
+				panel1.axhline(1000, color='red')
+			if chrom_based_IMD:
+				panel1.axhline(1000, color='red')					
+
+			ymax = 0
+			classColor = 0
+			firstClass = True
+			firstChrom = True 
+			chrom_prop_start = 0.075
+			for subclass in classes:
+				chrom_start = 0
+				if sample not in samples[subclass]:
+					classColor += 1
+					continue
+				for chrom in chroms:				
+					if firstClass:
+						centromere_start = centromeres[genome][chrom][0]
+						centromere_end = centromeres[genome][chrom][1]
+						centromere_startProp = ((chrom_start + centromere_start)/genomeLength) * 0.9 + 0.07
+						centromereLength = (centromere_end - centromere_start)/genomeLength * 0.9
+						chrom_prop_start = (chrom_start/genomeLength)*.9 + .07
+						chrom_prop = ((chromLengths[genome][chrom] + 10000)/genomeLength)*.9 -.003
+						panel1.add_patch(plt.Rectangle((chrom_prop_start, 0.06), chrom_prop, .01, facecolor="lightgrey", clip_on=False, transform=plt.gcf().transFigure)) 
+						panel1.add_patch(plt.Rectangle((centromere_startProp, 0.06), centromereLength, .01, facecolor="red", clip_on=False, transform=plt.gcf().transFigure)) 				
+						if chrom_based_IMD:
+							try:
+								imd_line = imdsData[sample.split("_")[0]][chrom]
+							except:
+								imd_line = imdsDataSample[sample.split("_")[0]]
+							panel1.plot([chrom_start, chrom_start+chromLengths[genome][chrom]],[imd_line,imd_line], color='red')
+
+					if chrom in allMutations[subclass].loc[sample].index:
+						pos = allMutations[subclass].loc[sample, chrom]
+
+						try:
+							starts = list(pos.loc[(sample, chrom), 'start'])
+							imds = [y-x for x,y in zip(starts, starts[1:])]
+							minIMDs = [min(x,y) for x,y in zip([float('inf')] + imds, imds + [float('inf')])]
+							if not aggregated:
+								if subclass == "Class III":
+									if chrom_based_IMD:
+										try:
+											plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]][chrom]]
+										except:
+											plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsDataSample[sample.split("_")[0]]]
+									else:
+										plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]]]
+								else:
+									plotIMDs = [[x,y] for x,y in zip(starts, minIMDs)]
+								plotX, plotY = zip(*plotIMDs)
+								newMax = max(plotY)
+								if chrom_based_IMD:
+									falsePositivesChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=imdsData[sample.split("_")[0]][chrom]]))
+									falsePositivesChrom_1000.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=1000]))
+								else:
+									falsePositivesChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=imdsData[sample.split("_")[0]]]))
+									falsePositivesChrom_1000.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=1000]))
+
+								try:
+									totalMutsIMDChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]][chrom]]))
+								except:
+									totalMutsIMDChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <= imdsDataSample[sample.split("_")[0]]]))
+								totalMuts1000Chrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <= 1000]))
+							else:
+								plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= 10000]
+								plotX, plotY = zip(*plotIMDs)
+								newMax = max(plotY)
+						except:
+							plotY = 0
+							try:
+								plotX = [int(pos.loc[(sample, chrom), 'start'])]
+							except:
+								plotY = [0 for x in pos.loc[(sample, chrom), 'start']]
+								plotX = [int(x) for x in pos.loc[(sample, chrom), 'start']]
+							newMax = 0
+						
+						if newMax > ymax:
+							ymax = newMax
+						plotX = [x + chrom_start for x in plotX]
+						densityMuts += plotX
+						panel1.scatter(plotX, plotY, color=colors[subclass], clip_on=False, s=12, label=subclass)
+					chrom_start += chromLengths[genome][chrom] + 10000
+				classColor += 1
+				firstClass = False
+			totalMutsIMD[sample] = sum(totalMutsIMDChrom)
+			totalMuts1000[sample] = sum(totalMuts1000Chrom)
+
+			falsePositives[sample.split("_")[0]].append(sum(falsePositivesChrom))
+			falsePositives_1000[sample.split("_")[0]].append(sum(falsePositivesChrom_1000))
+
+			hist, bins = np.histogram(densityMuts, bins=bins, density=True)
+			panel2.fill_between(bins[:-1], hist, zorder=10)
+			if ymax > 10**5:
+				panel1.set_ylim([1, ymax])
+			else:
+				panel1.set_ylim([1, 10**5])
+
+			panel1.set_yscale('log')
+			panel1.set_title("Clustered mutations - " + sample, pad=150, fontsize=20)
+			panel1.set_ylabel("Distance between mutations in a single event (log10)", fontsize=12)
+			panel1.spines['right'].set_visible(False)
+			panel1.spines['top'].set_visible(False)
+			panel1.spines['bottom'].set_visible(False)
+			panel1.set_xticks(xtick_pos)
+			panel1.set_xticklabels(["chr" + x for x in chroms], rotation=45)
+			panel1.set_xlim([10000, genomeLength])
+			panel2.set_xlim([10000, genomeLength])
+			panel2.set_ylabel("Density", labelpad=10)
+			panel1.tick_params(axis='y', which='major', labelsize=10)
+			panel1.tick_params(axis='x', which='major', labelsize=10, pad=20, length=0)
+			panel2.set_xticks([])
+			panel2.set_xticklabels([])
+			panel2.set_ylim([0, max(hist)*1.2])
+			panel2.grid(which='major', axis='y', color=[0.93,0.93,0.93], zorder=1)
+			handles, labels = panel1.get_legend_handles_labels()
+			goodLabels = [labels.index(x) for x in sorted(list(set(labels)))]
+			panel1.legend([handles[x] for x in goodLabels], [labels[x] for x in goodLabels], loc='best', prop={'size':10})
+			pp.savefig(plot1)
+			plt.close()
+
+		with open(projectPath_parent + "output/simulations/data/originalCounts" + path_suffix + ".pickle", "wb") as f:
+			pickle.dump(totalMutsIMD, f)
+		with open(projectPath_parent + "output/simulations/data/originalCounts1000" + path_suffix + ".pickle", "wb") as f:
+			pickle.dump(totalMuts1000, f)
+	else:
+		for chrom in chroms:
+			genomeLength = chromLengths[genome][chrom]
+			xtick_pos = []
+			bins = []
+			for i in range(0, genomeLength, windowSize):
+				bins.append(i)
+			bins.append(genomeLength)
+			count = 1
+			for sample in samplesSet:
+				densityMuts = []
+				count += 1
+				plot1 = plt.figure(figsize=(12,8))
+				plt.rc('axes', edgecolor='lightgray')
+				panel1 = plt.axes([0.07, 0.09, 0.9, 0.6])
+				panel2 = plt.axes([0.07, 0.75, 0.9, 0.15])
+				ymax = 0
+				classColor = 0
+				firstClass = True
+				firstChrom = True 
+				chrom_prop_start = 0.075
+				for subclass in classes:
+					chrom_start = 0
+					if sample not in samples[subclass]:
+						classColor += 1
+						continue
+					if firstClass:
+						centromere_start = centromeres[genome][chrom][0]
+						centromere_end = centromeres[genome][chrom][1]
+						centromere_startProp = (centromere_start/genomeLength) * 0.9 + 0.07
+						centromereLength = (centromere_end - centromere_start)/genomeLength * 0.9
+						chrom_prop_start = (chrom_start/genomeLength)*.9 + .07
+						chrom_prop = ((chromLengths[genome][chrom] + 10000)/genomeLength)*.9 -.003
+						panel1.add_patch(plt.Rectangle((chrom_prop_start, 0.06), chrom_prop, .01, facecolor="lightgrey", clip_on=False, transform=plt.gcf().transFigure)) 
+						panel1.add_patch(plt.Rectangle((centromere_startProp, 0.06), centromereLength, .01, facecolor="red", clip_on=False, transform=plt.gcf().transFigure)) 
+						
+					if chrom in allMutations[subclass].loc[sample].index:
+						pos = allMutations[subclass].loc[sample, chrom]
+						try:
+							starts = list(pos.loc[(sample, chrom), 'start'])
+							imds = [y-x for x,y in zip(starts, starts[1:])]
+							minIMDs = [min(x,y) for x,y in zip([float('inf')] + imds, imds + [float('inf')])]
+							if not aggregated:
+								if subclass == "Class III":
+									plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample]]
+								else:
+									plotIMDs = [[x,y] for x,y in zip(starts, minIMDs)]
+								plotX, plotY = zip(*plotIMDs)
+								newMax = max(plotY)
+							else:
+								plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= 10000]
+								plotX, plotY = zip(*plotIMDs)
+								newMax = max(plotY)
+
+						except:
+							plotY = 0
+							try:
+								plotX = [int(pos.loc[(sample, chrom), 'start'])]
+							except:
+								plotY = [0 for x in pos.loc[(sample, chrom), 'start']]
+								plotX = [int(x) for x in pos.loc[(sample, chrom), 'start']]
+							newMax = 0
+						
+						if newMax > ymax and newMax < float('inf'):
+							ymax = newMax
+						plotX = [x + chrom_start for x in plotX]
+						densityMuts += plotX
+						panel1.scatter(plotX, plotY, color=colors[subclass], clip_on=False, s=12, label=subclass)
+					chrom_start += chromLengths[genome][chrom] + 10000
+				classColor += 1
+				firstClass = False
+				hist, bins = np.histogram(densityMuts, bins=bins, density=True)
+				panel2.fill_between(bins[:-1], hist, zorder=10)
+				if ymax > 10**5:
+					panel1.set_ylim([1, ymax])
+				else:
+					panel1.set_ylim([1, 10**5])
+				panel1.set_yscale('log')
+				panel1.set_title("Clustered mutations chromosome " + chrom + " - " + sample, pad=150, fontsize=20)
+				panel1.set_ylabel("Distance between mutations in a single event (log10)", fontsize=12)
+				panel1.spines['right'].set_visible(False)
+				panel1.spines['top'].set_visible(False)
+				panel1.spines['bottom'].set_visible(False)
+				panel1.set_xticks(xtick_pos)
+				panel1.set_xticklabels(["chr" + x for x in chroms], rotation=45)
+				panel1.set_xlim([10000, genomeLength])
+				panel2.set_xlim([10000, genomeLength])
+				panel2.set_ylabel("Density", labelpad=10)
+				panel1.tick_params(axis='y', which='major', labelsize=10)
+				panel1.tick_params(axis='x', which='major', labelsize=10, pad=20, length=0)
+				panel2.set_xticks([])
+				panel2.set_xticklabels([])
+				try:
+					panel2.set_ylim([0, max(hist)*1.2])
+				except:
+					pass
+				panel2.grid(which='major', axis='y', color=[0.93,0.93,0.93], zorder=1)
+				handles, labels = panel1.get_legend_handles_labels()
+				goodLabels = [labels.index(x) for x in sorted(list(set(labels)))]
+				panel1.legend([handles[x] for x in goodLabels], [labels[x] for x in goodLabels], loc='best', prop={'size':10})
+				pp.savefig(plot1)
+				plt.close()
+	pp.close()
+
+
+
+
